#!/usr/bin/env python

import sys
import os
import ipaclib
try:
    import pygtk
    pygtk.require("2.0")
except:
    pass

try:
    import gtk
    import gtk.glade
except:
    print "You need to install pygtk and gtk V2"


class ipacgui:
    #classes
    keys = ipaclib.keylist()

    # Dialogs
    aboutdialog   = None
    opendialog    = None
    savedialog    = None
    debugdialog   = None
    changedialog  = None
    overwrtdialog = None
    

    # Widgets
    mainwindow = None
    statusbar = ""
    statusbar_ctx = ""
    control_notebook = ""
    macro_notebook = ""
    # The widget tree from Glade
    wTree = ""
    combo_model = ""
  
    # Entries in the device pane 
    rb_ipac2     = ""
    rb_ipac4     = ""
    rb_ipacve    = ""
    rb_minipacve = ""
    active_type  = "ipac2"

    # Images in the device pane
    img_ipac2    = ""
    img_ipac4    = ""
    img_ipacve    = ""
    img_minipacve    = ""

    # filenames for images
    fn_ipac2_not_det     = "ipac2_ndet.png"
    fn_ipac2_det         = "ipac2_det.png"
    fn_ipac4_not_det     = "ipac4_ndet.png"
    fn_ipac4_det         = "ipac4_det.png"
    fn_ipacve_not_det    = "ipacve_ndet.png"
    fn_ipacve_det        = "ipacve_det.png"
    fn_minipacve_not_det = "minipacve_ndet.png"
    fn_minipacve_det     = "minipacve_det.png"

    # Control notebook stuff
    control_nb = ""
    control_pages = []
    control_pages_shift = []
    control_widget_types = ["coin", "strt", "sw1", "sw2", "sw3", "sw4",
                            "sw5", "sw6", "sw7", "sw8", "a", "b", "c",
                            "d", "e", "f"]

    # Macro notebook stuff
    macro_nb = ""
    macro_pages = []

    ilib = ""
    debug_str = ""
    ipac_types = ["ipac2", "ipac4", "ipacve", "minipacve"]
    detected_ipacs = []

    # Current config file (+path) that is loaded
    current_filename = ""
    # clear the user edited flag to indicate config is unchanged
    useredited = False

    def get_widget(self, name):
        retval = None
        if self.wTree != "":
            retval = self.wTree.get_widget(name)
        return retval

    # User changed one of the combo boxes...
    def input_changed(self, widget):
        self.useredited = True
        title = self.main_window.get_title()
        if (title[-1:] != "*"):
            self.main_window.set_title(title + "*")

    def clear_changed(self):
        self.useredited = False
        title = self.main_window.get_title()
        if (title[-1:] == "*"):
            self.main_window.set_title(title[0:-1])
        self.dprint("clear_changed: edited flag cleared")

    def __init__(self):

        self.ilib = ipaclib.ipacutil()
        self.gladefile="ipacgui.glade"   
        winname = "MainWindow"
        self.wTree = gtk.glade.XML(self.gladefile, winname)
        signals = { 
                    "window_destroy"   : self.destroy,
                    "quit_clicked"     : self.destroy,
                    "open_clicked"     : self.open,
                    "save_clicked"     : self.save,
                    "save_as_clicked"  : self.save_as,
                    "new_clicked"      : self.new_click,
                    "detect_clicked"   : self.detect,
                    "program_clicked"  : self.program,
                    "debug_clicked"    : self.show_debug,
                    "shift_toggle"     : self.shift_toggled,
                    # One of the devices was selected
                    "rb_clicked"       : self.rb_click,
                    "help_about"       : self.about_click,
                    "inpt_changed"     : self.input_changed,
                  }

        self.wTree.signal_autoconnect(signals)
        self.main_window = self.wTree.get_widget(winname)

        self.control_notebook = self.wTree.get_widget('control_notebook')
        self.statusbar = self.wTree.get_widget('statusbar')
        self.statusbar_ctx = self.statusbar.get_context_id('status')

        self.rb_ipac2 = self.wTree.get_widget('rb_ipac2')
        self.rb_ipac4 = self.wTree.get_widget('rb_ipac4')
        self.rb_ipacve = self.wTree.get_widget('rb_ipacve')
        self.rb_minipacve = self.wTree.get_widget('rb_minipacve')
        self.show_all_devs(True)

        self.img_ipac2 = self.wTree.get_widget('img_ipac2')
        self.img_ipac4 = self.wTree.get_widget('img_ipac4')
        self.img_ipacve = self.wTree.get_widget('img_ipacve')
        self.img_minipacve = self.wTree.get_widget('img_minipacve')
        self.show_all_detected(False)

        self.control_nb = self.wTree.get_widget('control_notebook')
        self.control_pages.append(self.control_nb.get_nth_page(0))
        self.control_pages.append(self.control_nb.get_nth_page(1))
        self.control_pages.append(self.control_nb.get_nth_page(2))
        self.control_pages.append(self.control_nb.get_nth_page(3))
        self.control_pages_shift.append(self.control_nb.get_nth_page(4))
        self.control_pages_shift.append(self.control_nb.get_nth_page(5))
        self.control_pages_shift.append(self.control_nb.get_nth_page(6))
        self.control_pages_shift.append(self.control_nb.get_nth_page(7))
        self.control_nb.remove_page(7)
        self.control_nb.remove_page(6)
        self.control_nb.remove_page(5)
        self.control_nb.remove_page(4)

        self.macro_nb = self.wTree.get_widget('macro_notebook')
        for idx in range(0,8):
            self.macro_pages.append(self.macro_nb.get_nth_page(idx))

        # Make sure the correct pages are showing
        self.shift_btn = self.wTree.get_widget('shiftbtn')
        self.rb_ipac2.set_active(True)
        self.rb_click(self.rb_ipac2)

        self.init_all_comboboxes()

        # Find out what's attached
        gtk.main_iteration()
        self.detect(None)
        
        # Ensure we know that the config is unchanged
        self.clear_changed()
	return

    # Print to the status bar
    def sprint(self, text, debugtoo=0):
        if debugtoo:
            self.dprint(text)
        # Best effort to update statusbar.. if not, oh well
        try:
            self.statusbar.push(self.statusbar_ctx, text)
        except:
            pass

    # Print to the debug window
    def dprint(self, text):
        self.debug_str += text
        self.debug_str += "\n"

    # File | new clicked
    def new_click(self, widget):
        print "New clicked: edited: " + `self.useredited`
        if self.useredited == True:
            # Ask the user if they want to save
            self.changed_ask_save(widget)

        self.current_filename = ""
        self.clear_all_comboboxes()
        self.sprint("New configuration")

    def open(self, widget):
        self.dprint("open file")
        if (self.useredited):
            # ask user if they want to save their file
            self.changed_ask_save(widget)

        temp_wTree = gtk.glade.XML(self.gladefile, "dialog_fileopen")
        if self.opendialog == None:
            signals = {
                        "cancel_clicked" : self.open_dialog_close,
                        "open_clicked"   : self.open_dialog_open,
                      }
            self.opendialog = temp_wTree.get_widget("dialog_fileopen")
            temp_wTree.signal_autoconnect(signals)
        
        self.opendialog.show()
    
    def open_dialog_close(self, widget):
        self.opendialog.destroy()
        self.opendialog = None

    def open_dialog_open(self, widget):
        t_filename = self.opendialog.get_filename()
        if os.path.isdir(t_filename):
            # switch to dir
            self.opendialog.set_current_folder(t_filename)
        else:
            # New filename chosen to open
            self.current_filename = t_filename
            self.open_dialog_close(widget)
            self.sprint("Open: " + self.current_filename, 1)
            self.load_config_data(self.current_filename)

            # clear the user edited flag
            self.clear_changed()

    def shift_toggled(self, widget):
        idata = ipaclib.ipac(self.active_type)
        player_ct = idata.get_player_count()
        # Remove all the pages
        self.hide_all_control_pages()

        # And, add 'em back 
        self.dprint("Adding back " + `player_ct` + " player pages")
        self.dprint("Device type is: " + self.active_type)
        for idx in range(0, player_ct):
            self.show_control_page(idx+1, widget.get_active())

        # Now, do the same for the macro pages
        macro_ct = idata.get_macro_count()
        self.hide_all_macro_pages()
        self.dprint("Adding back " + `macro_ct` + " macro pages")
        for idx in range(0, macro_ct):
            self.show_macro_page(idx)

    # Returns True if overwrite chosen, else False
    def confirm_overwrite(self):
        retVal = False
        self.overwrite = ""

        temp_wTree = gtk.glade.XML(self.gladefile, "dialog_overwrite")
        if self.overwrtdialog == None:
            signals = {
                        "close_clicked" : self.overwrite_dialog_close,
                        "yesno_clicked" : self.overwrite_dialog_yesno,
                      }
            self.overwrtdialog = temp_wTree.get_widget("dialog_overwrite")
            temp_wTree.signal_autoconnect(signals)

        self.overwrtdialog.show()

        # Now, wait for a response
        while self.overwrite == "":
            gtk.main_iteration()

        self.dprint("overwrite response: " + `self.overwrite`)
        if self.overwrite:
            retVal = True

        return retVal

    def overwrite_dialog_close(self, widget):
        self.dprint("overwrite_dialog_close")
        # If the user just closes the window, assume that means no
        self.overwrite = False
        self.overwrite_dialog_dismiss()

    # Close the dialog window
    def overwrite_dialog_dismiss(self):
        self.overwrtdialog.destroy()
        self.overwrtdialog = None
  
    def overwrite_dialog_yesno(self, widget, response):
        if (response == gtk.RESPONSE_YES):
            self.overwrite = True
        if (response == gtk.RESPONSE_NO):
            self.overwrite = False
        self.overwrite_dialog_dismiss()

    def changed_ask_save(self, widget):
        temp_wTree = gtk.glade.XML(self.gladefile, "dialog_changed")
        if self.changedialog == None:
            signals = {
                        "close_clicked" : self.change_dialog_close,
                        "yesno_clicked" : self.change_dialog_yesno,
                      }
            self.changedialog = temp_wTree.get_widget("dialog_changed")
            temp_wTree.signal_autoconnect(signals)

        self.changedialog.show()

        # Now, wait for a response
        while self.useredited == True:
            gtk.main_iteration()
        self.dprint("changed_ask_save: Past the wait for ask save")

    def change_dialog_close(self, widget):
        self.changedialog.destroy()
        self.changedialog = None
  
    def change_dialog_yesno(self, widget, response):
        if (response == gtk.RESPONSE_YES):
            self.save(widget)
        if (response == gtk.RESPONSE_NO):
            self.sprint("Changed: Save cancelled by user", 1)
            # clear the user edited flag
            self.clear_changed()

        self.change_dialog_close(widget)


    def get_debug_text(self):
        newtext = self.debug_str 
        ipaclibtext = self.ilib.get_debug_buffer()
        if ipaclibtext != "":
            newtext += "===========\n==========\nFrom ipaclib:\n" + ipaclibtext
        return newtext

    def show_debug(self, widget):
        temp_wTree = gtk.glade.XML(self.gladefile, "dialog_debug")
        if self.debugdialog == None:
            signals = {
                        "close_clicked" : self.debug_dialog_close,
                        "save_clicked"  : self.debug_dialog_save,
                      }
            self.debugdialog = temp_wTree.get_widget("dialog_debug")
            temp_wTree.signal_autoconnect(signals)

        text_view = temp_wTree.get_widget("debug_text")
        textbuf = text_view.get_buffer()
        textbuf.set_text(self.get_debug_text())

        self.debugdialog.show()

    # Save contents of debug window to /tmp/ipacgui_debug.txt
    def debug_dialog_save(self, widget):
        try:
            dbg_file = "/tmp/ipacgui_debug.txt"
            fhdl = open(dbg_file, "w")
            fhdl.write(self.get_debug_text())
            fhdl.close()
            self.debug_dialog_close(widget)
            self.sprint("Debug info saved to " + dbg_file)
        except:
            pass
   
    def debug_dialog_close(self, widget):
        self.debugdialog.destroy()
        self.debugdialog = None

 

    def save_as(self, widget):
        #TODO: need a dialog for overwrite file
        self.dprint("Enter save_as")
        temp_wTree = gtk.glade.XML(self.gladefile, "dialog_filesave")
        if self.savedialog == None:
            signals = {
                        "cancel_clicked" : self.save_dialog_close,
                        "save_clicked"   : self.save_dialog_save,
                      }
            self.savedialog = temp_wTree.get_widget("dialog_filesave")
            temp_wTree.signal_autoconnect(signals)

        self.savedialog.show()
       
    def save_dialog_close(self, widget):
        try:
            self.savedialog.destroy()
        except:
            pass
        self.savedialog = None

    def save_dialog_save(self, widget):
        self.dprint("Enter save_dialog_save")
        t_filename = self.savedialog.get_filename()
        if os.path.isdir(t_filename):
            # switch to dir
            self.savedialog.set_current_folder(t_filename)
        else:
            # New filename chosen to open
            if os.path.exists(t_filename):
                # confirm overwrite
                overwt = self.confirm_overwrite()

            if overwt:
                self.current_filename = t_filename
                self.save_dialog_close(widget)
                self.save(widget)
            else:
                self.sprint("Save cancelled by user", 1)
                

    def save(self, widget):
        self.dprint("Enter save")
        if self.current_filename == "":
            self.save_as(widget)
        else:
            self.sprint("Saving to: " + self.current_filename, 1)

            self.dprint("save: gen config")
            self.generate_config_data(self.current_filename)

            self.dprint("save: clear flag")
            # clear the user edited flag
            self.clear_changed()

    def init_all_comboboxes(self):
        idata = ipaclib.all()

        for player in range(1,5):
            for shift in range(0,2):
                for ctrl in idata.get_player_inputs(player, shift):
                    #print "Ctrl: " + ctrl.lower()
                    combo_name = "cb_" + ctrl.lower()
                    if shift:
                        combo_name += "_s"
                    self.set_combobox_choices(combo_name)

        for ctrl in idata.get_macro_inputs():
            combo_name = "cb_" + ctrl.lower()
            self.set_combobox_choices(combo_name)
     
        combo_name = "cb_shift"
        self.set_combobox_choices(combo_name)

    # Builds up the combobox model -- common for all
    def build_combo_model(self):
        nonekey = -1 
        idx = 0
        self.combo_model = gtk.ListStore(str)
        for keycode in self.keys.get_keys():
            if (keycode == "NONE"):
                nonekey = idx
            self.combo_model.append([keycode])
            idx += 1
        return nonekey


    # Create the combo box choices for each combo box
    def set_combobox_choices(self, combo_name):
        if self.combo_model == "":
            self.nonekey = self.build_combo_model()

        combo = self.get_widget(combo_name)
        combo.set_model(self.combo_model)

        cell = gtk.CellRendererText()
        combo.pack_start(cell)
        combo.add_attribute(cell, 'text', 0)
        combo.set_active(self.nonekey)
        
    def clear_all_comboboxes(self): 
        idata = ipaclib.all()

        for player in range(1,5):
            for shift in range(0,2):
                if (shift):
                    suffix = "_s"
                else:
                    suffix = ""
                for ctrl in idata.get_player_inputs(player, shift):
                    #print "Ctrl: " + ctrl.lower()
                    combo_name = "cb_" + ctrl.lower() + suffix
                    combo = self.get_widget(combo_name)
                    combo.set_active(self.nonekey)

        # clear the user edited flag
        self.clear_changed()

    def control_widget_name(self, player, input, shift):
        retval = `player` + input
        if (shift):
            retval = retval + "_s"
        return retval
        

    def config_control_pages(self, type, shift):
        idata = ipaclib.ipac(type)
        for player in range(1, idata.get_player_count()+1):
            for shift in range(0, 2):
                for wid_type in self.control_widget_types:
                    name_suffix = self.control_widget_name(player, wid_type, shift)
                    combo_name = "cb_" + name_suffix
                    label_name = "lbl_" + name_suffix
                    image_name = "img_" + name_suffix
                    combo = self.get_widget(combo_name)
                    if (combo == None):
                        self.dprint("Couldn't find " + combo_name)
                        # Fail somewhat gracefully?
                        return
                    label = self.get_widget(label_name)
                    image = self.get_widget(image_name)

                    if idata.has_input(wid_type, shift):
                        #print "Show: " + combo_name
                        combo.show()
                        label.show()
                        image.show()
                    else:
                        #print "Hide: " + label_name
                        combo.hide()
                        label.hide()
                        image.hide()

        # TODO: configure the macro inputs


    # Detect connected I-PACs
    def detect(self, widget):
        #self.show_all_devs(False)
        self.show_all_detected(False)
        ilist = self.ilib.detect_ipacs()
        self.dprint("detect: found: " + `ilist`)
        # ilist is list of lists -- each entry is ipac num, ipac name
        for ipac in ilist:
            self.sprint(ipac[1] + " detected")
            #self.show_device(ipac[1], True)
            self.show_detected(ipac[1], True)
            self.detected_ipacs.append(ipac)
        if (len(ilist) == 0):
            self.sprint("No I-PACs detected", 1)

    # Check if the selected ipac is in the detected list
    # If present, returns board number
    # if not present, returns -1
    def is_selected_ipac_present(self):
        sel_type = -1
        board_num = -1
        if self.rb_ipac2.get_active():
            # TODO: type should be a constant from ipaclib
            sel_type = "ipac2"
        elif self.rb_ipac4.get_active():
            sel_type = "ipac4"
        elif self.rb_ipacve.get_active():
            sel_type = "ipacve"
        elif self.rb_minipacve.get_active():
            sel_type = "minipacve"

        self.dprint("is_selected_ipac_present: found: " + sel_type)

        for detected in self.detected_ipacs:
            if sel_type == detected[1]:
                board_num = detected[0]
                break

        self.dprint("is_selected_ipac_present: num: " + `board_num`)
        return board_num

    def rb_click(self, widget):
        self.dprint("enter rb_click")
        active = widget.get_active()
        wid_name = widget.get_name()
        if wid_name == "rb_ipac2":
            type = "ipac2"
        elif wid_name == "rb_ipac4":
            type = "ipac4"
        elif wid_name == "rb_ipacve":
            type = "ipacve"
        elif wid_name == "rb_minipacve":
            type = "minipacve"

        if active:
            self.active_type = type
            idata = ipaclib.ipac(type)
            player_ct = idata.get_player_count()
            self.sprint(type + " selected: " + `player_ct` + " players", 1)

            # Display the appropriate control pages
            self.shift_toggled(self.shift_btn)
            # With the appropriate content
            self.config_control_pages(type, False)



    def show_device(self, type, show):
        widget = ""
        if (type == "ipac2"):
            widget = self.rb_ipac2
        elif (type == "ipac4"):
            widget = self.rb_ipac4
        elif (type == "ipacve"):
            widget = self.rb_ipacve
        elif (type == "minipacve"):
            widget = self.rb_minipacve

        if (widget != ""):
            if (show): 
                widget.show()
            else:
                widget.hide()

    def show_all_devs(self, show):
        for ipac in self.ipac_types:
            self.show_device(ipac, show)

    def show_all_detected(self, detected):
        for ipac in self.ipac_types:
            self.show_detected(ipac, detected)

    def show_detected(self, type, detected):
        self.dprint("show_detected: " + type + " detected: " + `detected`);
        widget = ""
        if (type == "ipac2"):
            if detected:
                self.img_ipac2.set_from_file(self.fn_ipac2_det)
            else:
                self.img_ipac2.set_from_file(self.fn_ipac2_not_det)
        elif (type == "ipac4"):
            if detected:
                self.img_ipac4.set_from_file(self.fn_ipac4_det)
            else:
                self.img_ipac4.set_from_file(self.fn_ipac4_not_det)
        elif (type == "ipacve"):
            if detected:
                self.img_ipacve.set_from_file(self.fn_ipacve_det)
            else:
                self.img_ipacve.set_from_file(self.fn_ipacve_not_det)
        elif (type == "minipacve"):
            if detected:
                self.img_minipacve.set_from_file(self.fn_minipacve_det)
            else:
                self.img_minipacve.set_from_file(self.fn_minipacve_not_det)

    def show_macro_page(self, num):
        mpage = self.macro_pages[num]
        label = gtk.Label("Macro " + `num + 1`)
        self.macro_nb.insert_page(mpage, label, num)

    def show_control_page(self, player, shifted):
        if (shifted):
            cpage = self.control_pages_shift[player-1]
        else:
            cpage = self.control_pages[player-1]

        #print "Show page " + `player` + " shifted: " + `shifted`
        label = gtk.Label("Player " + `player`)
        self.control_nb.insert_page(cpage, label, player-1)

    def hide_all_control_pages(self):
        for idx in range(0, self.control_nb.get_n_pages()):
            self.control_nb.remove_page(0)

    def hide_all_macro_pages(self):
        for idx in range(0, self.macro_nb.get_n_pages()):
            self.macro_nb.remove_page(0)

    def destroy(self, widget):
        if self.useredited == True:
            # Ask the user if they want to save
            self.changed_ask_save(widget)

        gtk.main_quit()

    def program(self, widget):
        if self.current_filename == "":
            # Set edited flag to force wait for save dialog
            self.useredited = True

        if (self.current_filename == "") or (self.useredited):
            self.changed_ask_save(widget)

        if (self.current_filename == ""):
            self.sprint("Programming cancelled -- config file must be saved", 1)
            return

        board_num = self.is_selected_ipac_present()
        if board_num < 0:
            self.sprint("Programming cancelled -- selected I-PAC is not detected", 1)
            return

        self.sprint("Programming the I-PAC...", 1)
        #TODO:add a RAM-only flag somewhere
        res = self.ilib.program(self.current_filename, board_num, False)
        if res == 1:
            self.sprint("I-PAC sucessfully programmed", 1)
        else:
            self.sprint("ERROR: Programming I-PAC failed", 1)
            

    # Load an ipacutil config file and populate the comboboxes
    def load_config_data(self, filename):
        self.dprint("Load config data from: " + filename)
        cfgdata = self.ilib.read_config_file(filename)
        # Should be 5 sections: shift code, board_type, unshifted, shifted, macro
        if len(cfgdata) != 5:
            # Invalid data file
            self.sprint("Invalid data file: " + filename, 1)
            self.current_filename = ""
        else:
            # Valid data file -- load comboboxes
            bdtype_off = self.ilib.rc_boardtype_offset
            bdtype = cfgdata[bdtype_off][0]

            # Shift code
            self.dprint("Shift Code: " + `cfgdata[0]`)
            combo = self.get_widget("cb_shift")
            res_off = self.ilib.rc_shiftcode_offset
            keyname = self.keys.get_keyname(cfgdata[res_off][0])
            combo.set_active(self.keys.get_keyname_offset(keyname))

            # Player inputs -- unshifted and shifted
            idata = ipaclib.ipac(bdtype)
            player_ct = idata.get_player_count()
            for shift in range(0, 2):
                if (shift == 0):
                    # Unshifted data
                    secdata = cfgdata[self.ilib.rc_unshifted_offset]
                    self.dprint("Loading unshifted block")
                    combo_suffix = ""
                else:
                    # Shifted data
                    secdata = cfgdata[self.ilib.rc_shifted_offset]
                    self.dprint("Loading shifted block")
                    combo_suffix = "_s"

                # This includes inputs for all players
                inputs = idata.get_input_list(shift)
                idx = 0
                for inpt in inputs:
                    # D'oh... numbering is different in app vs board
                    # simpler to keep format in program and fix up here
                    # TODO: put this in a function -- it's used for writing the config too
                    if (inpt[0:1] > "0") and (inpt[0:1] < "5"):
                        combo_name = "cb_" + inpt.lower()
                    else:
                        combo_name = "cb_" + inpt[-1:] + inpt[0:-1]
                        combo_name = combo_name.lower()

                    combo_name += combo_suffix
                    self.dprint("Player input: combo: " + combo_name)
                    combo = self.get_widget(combo_name)
                    self.dprint("idx: " + `idx`)
                    keyname = self.keys.get_keyname(secdata[idx][0])
                    self.dprint("keyname: " + keyname)
                    combo.set_active(self.keys.get_keyname_offset(keyname))
                    idx += 1

            # Macros
            macrodata = cfgdata[self.ilib.rc_macro_offset]
            macroinputs = idata.get_macro_inputs()
            self.dprint("Macro Data from Config File: len: " + `len(macrodata)`)
            self.dprint("length from get_macro_inputs: " + `len(macroinputs)`)
            if len(macrodata) != len(macroinputs):
                self.sprint("Invalid macro section -- ignored", 1)
                return
            idx = 0
            for mac in macroinputs:
                combo_name = "cb_" + mac.lower()
                self.dprint("idx: %d, combo: %s" % (idx, combo_name))
                combo = self.get_widget(combo_name)
                keyname = self.keys.get_keyname(macrodata[idx][0])
                #print "keyname: " + keyname
                combo.set_active(self.keys.get_keyname_offset(keyname))
                idx += 1

            # And update board type and views
            self.dprint("Setting board type to %s from config file" % bdtype)
            self.select_device(bdtype)
            self.sprint("Config data loaded from " + filename, 1)
        

    # Programmatically change the selected device
    def select_device(self, type):
        self.dprint("select_device: " + type)
        rb_wid = None
        if (type == "ipac2"):
           rb_wid = self.rb_ipac2
        elif type == "ipac4":
           rb_wid = self.rb_ipac4
        elif type == "ipacve":
           rb_wid = self.rb_ipacve
        elif type == "minipacve":
           rb_wid = self.rb_minipacve

        self.dprint("widget chosen:" + `rb_wid`)
        rb_wid.set_active(True)
        self.rb_click(rb_wid)


    # Save the current state to an ipacutil-style config file
    def generate_config_data(self, filename):
        idata = ipaclib.ipac(self.active_type)
        unshifted_data = []
        shifted_data = []
        for shift in range(0,2):
            if (shift):
                dlist = shifted_data
            else:
                dlist = unshifted_data

            for input in idata.get_input_list(shift):
                # D'oh... numbering is different in app vs board
                # simpler to keep format in program and fix up here
                if (input[0:1] > "0") and (input[0:1] < "5"):
                    combo_name = "cb_" + input.lower()
                else:
                    combo_name = "cb_" + input[-1:] + input[0:-1] 
                    combo_name = combo_name.lower()
                if shift:
                    combo_name += "_s"
                self.dprint("Gen:Combo name: " + combo_name)
                combo = self.get_widget(combo_name)
                active = combo.get_active()
                selection = self.keys.get_keys()[active]
                self.dprint("Gen:Selection: %s" % selection)
                code = self.keys.get_keycode(selection)
                dlist.append([hex(code), input + " - " + selection])
                self.dprint("Gen:Code is: 0x%02x" % code)

        # get shift code
        combo = self.get_widget("cb_shift")
        selection = self.keys.get_keys()[combo.get_active()]
        code = self.keys.get_keycode(selection)
        shift_code = [hex(code), selection]

        # Build the macro data
        macro_data = []
        for mac in idata.get_macro_inputs():
            combo_name = "cb_" + mac.lower()
            combo = self.get_widget(combo_name)
            selection = self.keys.get_keys()[combo.get_active()]
            code = self.keys.get_keycode(selection)
            macro_data.append([hex(code), mac + " - " + selection])

        res = self.ilib.write_config_file(filename, self.active_type, 
                                          shift_code, unshifted_data, 
                                          shifted_data, macro_data)
        if res == 0:
            self.sprint("Failed to save " + filename, 1)
        else:
            self.sprint("Saved to " + filename, 1)
                
      
    def about_click(self, widget):
        temp_wTree = gtk.glade.XML(self.gladefile, "dialog_about")
        if self.aboutdialog == None:
            signals = {
                        "about_close" : self.about_close,
                      }
            self.aboutdialog = temp_wTree.get_widget("dialog_about")
            self.aboutdialog.set_name("ipacgui")
            temp_wTree.signal_autoconnect(signals)
        
        self.aboutdialog.show()

    def about_close(self, dialog, params):
        self.aboutdialog.destroy()
        self.aboutdialog = None

app = ipacgui()
gtk.main()
