#!/usr/bin/env python

import string, os, re

class ipacutil:
   
    # 'constants' 
    cmd_which     = 'which'
    cmd_ipacutil  = 'ipacutil'
    flag_detect   = ' -d'
    flag_reset    = ' -r'
    flag_ram      = ' -a'
    flag_write    = ' -w '
    flag_board    = ' -b '
    flag_config   = ' -c '
    flag_type     = ' -t '
    board_re      = 'I-PAC Board Number ([0-9]): ([a-z]+[0-9]*)'
    write_re      = 'Created new config file ([^ ]+) for board type ([a-z]+[0-9]*)'
    pgm_re        = '\*\* I-PAC has been reprogrammed \*\*'
    section_misc       = '[Misc]'
    section_unshifted  = '[Unshifted]'
    section_shifted    = '[Shifted]'
    section_macro      = '[Macro]'
    section_unshifted2 = '[Unshifted2]'
    section_shifted2   = '[Shifted2]'
    section_macro2     = '[Macro2]'
    section_close      = '.\n'

    section_name_misc       =  "Misc"
    section_name_unshifted  = "Unshifted"
    section_name_unshifted2 = "Unshifted2"
    section_name_shifted    = "Shifted"
    section_name_shifted2   = "Shifted2"
    section_name_macro      = "Macro"
    section_name_macro2     = "Macro2"

    varname_shiftcode  = "ShiftCode"
    varname_boardtype  = "BoardType"

    rc_shiftcode_offset = 0
    rc_boardtype_offset = 1
    rc_unshifted_offset = 2
    rc_shifted_offset   = 3
    rc_macro_offset     = 4

    # Board number
    board_dict        = {'ipac2':0,  'ipac4':1,  'ipacve':2,  'minipacve':3}
    board_type_dict   = {0:'ipac2',  1:'ipac4',  2:'ipacve',  3:'minipacve'}
    # Section size - unshifted/shifted
    ssec_dict    = {'ipac2': 28, 'ipac4': 28, 'ipacve': 32, 'minipacve': 36}
    usec_dict    = {'ipac2': 28, 'ipac4': 28, 'ipacve': 28, 'minipacve': 28}
    # Macro size
    macro_dict   = {'ipac2': 16, 'ipac4': 16, 'ipacve': 16, 'minipacve': 16}
    # Num Passes
    pass_dict    = {'ipac2': 1,  'ipac4': 2,  'ipacve': 1,  'minipacve': 1}

    ipacutil_path = ''
    ipac_list     = []

    debug_str = ""


    ####################################
    # Constructor
    ####################################
    def __init__(self):
        if (self.find_ipacutil() == ''):
           raise 'ipacutil not found in path'

    ####################################
    # Debug print - print to debug buffer
    ####################################
    def dprint(self, text):
        self.debug_str += text
        self.debug_str += "\n"

    ####################################
    # Get the debug text from operations thus far
    ####################################
    def get_debug_buffer(self):
        return self.debug_str

    ####################################
    # Find the path to ipacutil
    # Returns path to ipacutil
    ####################################
    def find_ipacutil(self):

        #TODO: might want to consider using whereis instead of which...
        if self.ipacutil_path == '':    
            for file in os.popen(self.cmd_which + ' ' + self.cmd_ipacutil).readlines():
                self.ipacutil_path = file.rstrip()

        return self.ipacutil_path;

    ####################################
    # Returns a list of lists.  Each entry contains the board number and board type
    ####################################
    def detect_ipacs(self):
        cmd = self.find_ipacutil() + self.flag_detect
        self.ipac_list = []
 
        entry_list = []

        #boardPatn = re.compile(self.board_re)
        for line in os.popen(cmd).readlines():
            self.dprint("detect_ipacs: line: " + line)
            res = re.match(self.board_re, line)
            if res:
                entry_list = []
                if len(res.groups()) == 2:
                    # The board number
                    entry_list.append(res.group(1))
                    # The board type
                    entry_list.append(res.group(2))
                    self.ipac_list.append(entry_list)

        return self.ipac_list
  
    ####################################
    # Write the default config file...
    # Returns 0 on fail, else 1
    ####################################
    def write_default_config(self, outfile, type):
        cmd = self.find_ipacutil() + self.flag_type + type + self.flag_write + outfile
        retval = 0
        
        for line in os.popen(cmd).readlines():
            self.dprint("write_default_config: line: " + line)
            res = re.match(self.write_re, line)
            if res:
                retval = 1

        return retval

    # Write a string to a file as a line
    def writeline(self, fhdl, str):
	fhdl.write(str + '\n')

    ####################################
    # Generate a custom data file
    # shift_code should be a list of 2 items: code and comment
    # unshifted/shifted should be lists of 2 item lists: code and comment
    # macro should be a list of 2 item lists: code and comment
    # Returns 1 on good status, else 0
    ####################################
    def write_config_file(self, outfile, type, shift_code, unshifted, shifted, macro):
        retval = 0

        if (self.board_dict.has_key(type) == False):
            self.dprint('write_config_file:Invalid board type')
            return retval


        ssec_len = self.ssec_dict[type]
        usec_len = self.usec_dict[type]
        passes  = self.pass_dict[type]
        if ((len(unshifted) != usec_len * passes) or 
            (len(shifted)   != ssec_len * passes)):

            # invalid section length
            self.dprint('write_config_file:Invalid section length: ')
            self.dprint('write_config_file:  unshifted len: ' + str(len(unshifted)) + ' expected: ' + str(usec_len * passes))
            self.dprint('write_config_file:  shifted len: ' + str(len(shifted)) + ' expected: ' + str(ssec_len * passes))
            return retval

        macro_len = self.macro_dict[type]
        if (macro_len * passes != len(macro)):
            self.dprint('Invalid macro section length: ' + str(macro_len))
            self.dprint('  macro len: ' + str(len(macro)) + ' expected: ' + str(macro_len * passes))
            return retval

        try:
            fhdl = open(outfile, 'w')
        except:
            return retval

        # Misc section
	self.writeline(fhdl, self.section_misc)
        self.writeline(fhdl, 'ShiftCode\t' + shift_code[0] + '\t;' + shift_code[1])
        self.writeline(fhdl, 'BoardType\t' + str(self.board_dict[type]) + '\t;board type - ' + type);
        self.writeline(fhdl, self.section_close)

        for passnum in range(0, passes):
            # Unshifted section
            if (passnum == 0):
               sec = self.section_unshifted
            else: 
               sec = self.section_unshifted2
            self.writeline(fhdl, sec)
            for idx in range(0, usec_len):
                offset = passnum * usec_len + idx
                item = unshifted[offset]
                self.writeline(fhdl, str(idx) + '\t' + item[0] + '\t;' + item[1])
            self.writeline(fhdl, self.section_close)

            # Shifted section
            if (passnum == 0):
               sec = self.section_shifted
            else: 
               sec = self.section_shifted2
            self.writeline(fhdl, sec)
            for idx in range(0, ssec_len):
                offset = passnum * ssec_len + idx
                item = shifted[offset]
                self.writeline(fhdl, str(idx) + '\t' + item[0] + '\t;' + item[1])
            self.writeline(fhdl, self.section_close)

            # Macro section
            if (passnum == 0):
               sec = self.section_macro
            else: 
               sec = self.section_macro2
            self.writeline(fhdl, sec)
            for idx in range(0, macro_len):
                offset = passnum * macro_len + idx
                item = macro[offset]
                self.writeline(fhdl, str(idx) + '\t' + item[0] + '\t;' + item[1])
            self.writeline(fhdl, self.section_close)

        fhdl.close()
        retval = 1

        return retval

    ####################################
    # Program the ipac
    # Returns 0 on fail, else 1
    ####################################
    def program(self, config_file, board_num, ram_only):
        cmd = self.find_ipacutil() + self.flag_board + str(board_num) 
        cmd += self.flag_config + config_file
        if ram_only:
            cmd += self.flag_ram

        retval = 0

        for line in os.popen(cmd).readlines():
            self.dprint("program: line: " + line)
            res = re.search(self.pgm_re, line)
            if res:
                retval = 1

        return retval

    ####################################
    # Reset to factory defaults
    # Returns 0 on fail, else 1
    ####################################
    def reset(self, board_num):
        retval = 0
        cmd = self.find_ipacutil() + self.flag_board + str(board_num) + self.flag_reset
    
        for line in os.popen(cmd).readlines():
            self.dprint("reset: line: " + line)
            res = re.search(self.pgm_re, line)
            if res:
                retval = 1

        return retval

    # Try to be smart about converting a number -- handle either
    # decimal of hex (with leading 0x)
    def str_to_int(self, numstr):
        base = 10

        if (numstr[0:2].upper() == "0X"):
            base = 16;
        return int(numstr, base)

  

    ####################################
    # Try to read a config file
    # Returns an empty list on error
    # On success, returns a list of lists:
    #  idx 0: shift_code
    #  idx 1: type (string)
    #  idx 2: unshifted data block
    #  idx 3: shifted data block
    #  idx 4: macro data block
    ####################################
    def read_config_file(self, cfgfile):
        retList = []
        shift_code = "0xff"
        type = ""
        unshifted = []
        shifted = []
        macro = []

        section_re = "^\[(.*)\]"
        section_cre = re.compile(section_re)

        data_re = "^([0-9]+)[ \t]+(0x[0-9a-fA-F]+)[ \t]+;(.*)"
        data_cre = re.compile(data_re)
   
        var_re = "^([A-Za-z]+)[ \t]+([0x]*[0-9a-fA-F]+)[ \t]+;(.*)"
        var_cre = re.compile(var_re)

        cur_section = ""

        try:
            fhdl = open(cfgfile, 'r')
        except:
            self.dprint("Failed to open: %s" % cfgfile)
            return retList

        for line in fhdl.readlines():
            if (line[0:1] == "#"):
                # comment
                continue

            res = section_cre.search(line)
            if (res):
                # Section open
                cur_section = res.groups()[0]
                self.dprint("Current section: %s" % cur_section)
                continue

            if (line[0:1] == "."):
                # Section close
                cur_section = ""
                self.dprint("Exit section")
                continue

            # Ignore all data not in a section
            if cur_section == "":
                continue

            res = data_cre.search(line)
            if (res):
                self.dprint("Found data line: %s" % line)
                self.dprint("Current section: %s" % cur_section)
                # Data line in a section
                if ((cur_section == self.section_name_unshifted) or
                   (cur_section == self.section_name_unshifted2)):
                    #self.dprint("Adding an unshifted def")
                    #self.dprint("Arg1: " + `res.groups()[1]`)
                    #self.dprint("Arg2: " + `res.groups()[2]`)
                    unshifted.append([res.groups()[1], res.groups()[2]])
                elif ((cur_section == self.section_name_shifted) or
                     (cur_section == self.section_name_shifted2)):
                    shifted.append([res.groups()[1], res.groups()[2]])
                elif ((cur_section == self.section_name_macro) or
                     (cur_section == self.section_name_macro2)):
                    #self.dprint("Adding a macro def")
                    #self.dprint("Arg1: " + `res.groups()[1]`)
                    #self.dprint("Arg2: " + `res.groups()[2]`)
                    macro.append([res.groups()[1], res.groups()[2]])
                continue
 
            if (cur_section != self.section_name_misc):
                # Nothing more to do if not in misc section
                continue
 
            res = var_cre.search(line)
            if (res):
                # var line in Misc section
                varname = res.groups()[0]
                self.dprint("Found var: %s" % varname )
                if (varname == self.varname_shiftcode):
                    self.dprint("Found shift code")
                    shift_code = [res.groups()[1], res.groups()[2]]
                elif (varname == self.varname_boardtype):
                    self.dprint("Found board type")
                    try:
                        typenum = self.str_to_int(res.groups()[1])
                    except:
                        typenum = -1
                    if self.board_type_dict.has_key(typenum):
                        typename = self.board_type_dict[typenum]
                        type = [typename, res.groups()[2]]
                continue

        fhdl.close()
 
        self.dprint("Shift code: %s" % `shift_code`)
        self.dprint("type : %s" % `type`)
        self.dprint("Macro: %s" % `macro`)
        if (len(shift_code) < 1):
            self.dprint("ERROR: empty shift code\n");
        if (len(type) < 1):
            self.dprint("ERROR: empty board type\n");
        if (len(unshifted) < 1):
            self.dprint("ERROR: empty unshifted section\n");
        if (len(shifted) < 1):
            self.dprint("ERROR: empty shifted section\n");
        if (len(macro) < 1):
            self.dprint("ERROR: empty macro section\n");
        if (type != ""):
            retList.append(shift_code)
            retList.append(type)
            retList.append(unshifted)
            retList.append(shifted)
            retList.append(macro)
        return retList

    ####################################
    # Key list -- all 104 keys from PC keyboard
    #TODO: is there any point in having this?  see keys.py
    keylist = [
        ["A", 0x1C],
        ["B", 0x32],
        ["C", 0x21],
        ["D", 0x23],
        ["E", 0x24],
        ["F", 0x2B],
        ["G", 0x34],
        ["H", 0x33],
        ["I", 0x43],
        ["J", 0x3B],
        ["K", 0x42],
        ["L", 0x4B],
        ["M", 0x3A],
        ["N", 0x31],
        ["O", 0x44],
        ["P", 0x4D],
        ["Q", 0x15],
        ["R", 0x2D],
        ["S", 0x1B],
        ["T", 0x2C],
        ["U", 0x3C],
        ["V", 0x2A],
        ["W", 0x1D],
        ["X", 0x22],
        ["Y", 0x35],
        ["Z", 0x1A],
        ["0", 0x45],
        ["1", 0x16],
        ["2", 0x1E],
        ["3", 0x26],
        ["4", 0x25],
        ["5", 0x2E],
        ["6", 0x36],
        ["7", 0x3D],
        ["8", 0x3E],
        ["9", 0x46],
        ["`", 0x0E],
        ["-", 0x4E],
        ["=", 0x55],
        [",", 0x41],
        ["\\", 0x5D],
        ["BKSP", 0x66],
        ["SPACE", 0x29],
        ["TAB", 0x0D],
        ["CAPS", 0x58],
        ["LSHFT", 0x12],
        ["LCTRL", 0x14],
        ["LGUI", 0xE0],
        ["LALT", 0x11],
        ["RSHFT", 0x59],
        ["RCTRL", 0xE0],
        ["RGUI", 0xE0],
        ["RALT", 0xE0],
        ["APPS", 0xE0],
        ["ENTER", 0x5A],
        ["ESC", 0x76],
        ["F1", 0x05],
        ["F2", 0x06],
        ["F3", 0x04],
        ["F4", 0x0C],
        ["F5", 0x03],
        ["F6", 0x0B],
        ["F7", 0x83],
        ["F8", 0x0A],
        ["F9", 0x01],
        ["F10", 0x09],
        ["F11", 0x78],
        ["F12", 0x07],
        ["PRNTSCRN", 0xE0],
        ["SCROLL", 0x7E],
        ["PAUSE", 0xE1],
        ["[", 0x54],
        ["INSERT", 0xE0],
        ["HOME", 0xE0],
        ["PGUP", 0xE0],
        ["DELETE", 0xE0],
        ["END", 0xE0],
        ["PGDN", 0xE0],
        ["UARROW", 0xE0],
        ["LARROW", 0xE0],
        ["DARROW", 0xE0],
        ["RARROW", 0xE0],
        ["NUM", 0x77],
        ["KP/", 0xE0],
        ["KP*", 0x7C],
        ["KP-", 0x7B],
        ["KP+", 0x79],
        ["KPEN", 0xE0],
        ["KP.", 0x71],
        ["KP0", 0x70],
        ["KP1", 0x69],
        ["KP2", 0x72],
        ["KP3", 0x7A],
        ["KP4", 0x6B],
        ["KP5", 0x73],
        ["KP6", 0x74],
        ["KP7", 0x6C],
        ["KP8", 0x75],
        ["KP9", 0x7D],
        ["]", 0x5B],
        [";", 0x4C],
        ["'", 0x52],
        [".", 0x49],
        ["/", 0x4A]]
        

##############################################3
#Test
#ipac = ipacutil()
#print 'Ipacutil is ' + ipac.find_ipacutil()
#boards = ipac.detect_ipacs()
#boardct = len(boards)
#print 'Found ' + str(boardct) + ' boards'

#if (boardct < 1):
#   raise 'no ipacs'

#print 'first board: ' + str(boards[0][0]) + ' type: ' + boards[0][1]

#cfg_file = '/tmp/ipac.conf'
#if (ipac.write_default_config(cfg_file, boards[0][1])):
#    print 'Wrote config file: ' + cfg_file
#else:
#    print 'Failed to write config file: ' + cfg_file

#if (ipac.program('/tmp/ipac.conf', boards[0][0], 1)):
#    print 'Successfully programmed'
#else:
#    print 'programming failed'

#if (ipac.reset(boards[0][0])):
#    print 'Successfully reset'
#else:
#    print 'Reset failed'

#unshifted = []
#for idx in range(0,56):
#   unshifted.append([str(idx), 'comment ' + str(idx)])

#shifted = unshifted

#macro = []
#for idx in range(0,32):
#    macro.append([str(idx), 'comment ' + str(idx)])

#ipac.write_config_file('/tmp/myipac.conf', 'ipac4', ['0x10', 'tab?'], unshifted, shifted, macro)

#print ipac.keylist
#print "0x%X" % ipac.keylist[0][1]

##########################################################################
# I-PAC Data Classes - description of each model I-PAC
# Can instantiate each separately ipac2, ipac4, ipacve, minipacve, all
# or an ipac class which handles the details:
#     idata = ipac('ipac2')
##########################################################################

# Available models (classes): ipac2, ipac4, ipacve, minipacve

class ipac_data:

    # Return the number of player inputs this model has
    def get_player_count(self):
        return self.player_count

    # Return the number of macro inputs (each macro is up to 4 keystrokes)
    def get_macro_count(self):
        return len(self.macro_inputs) / 4

    # Returns the model name string
    def get_model_name(self):
        return self.model_name

    # Returns the model number
    def get_model_num(self):
        return self.model_num

    # Returns 1 if has, else 0
    def has_input(self, input, shifted):
        if shifted:
            inp_list = self.player_inputs_shift
        else:
            inp_list = self.player_inputs

        try:
            idx = inp_list.index(input.upper())
        except:
            idx = 0

        return idx
            


    # Return the list of inputs for the player specified
    # if shifted is True, returns the list for shifted mode
    def get_player_inputs(self, player, shifted):
        ret_list = []
       
        if (player <= self.player_count):
            if (shifted):
                dlist = self.player_inputs_shift
            else:
                dlist = self.player_inputs
            for item in dlist:
                ret_list.append(str(player) + item)

        return ret_list

    def get_macro_inputs(self):
        return self.macro_inputs

    # Returns the offset into the data block for a specified input name
    # IE  1SW3 maps to idx 5
    # Returns -1 on error
    def get_input_index(self, input_name, shifted):
        retval = -1

        if shifted:
            map = self.map_shifted
        else:
            map = self.map_unshifted

        if (map.has_key(input_name.upper())):
            retval = map[input_name.upper()]

        return retval

    # Return the input list, in the order of the config file
    def get_input_list(self, shifted):
        if shifted:
            map = self.map_shifted
        else:
            map = self.map_unshifted

        # Get the list in the right order
        tlist = []
        for key in map.keys():
            tlist.append([map[key], key])
        
        tlist.sort()

        # Now, split off just the key
        retlist = []
        for item in tlist:
            retlist.append(item[1])

        return retlist


# IPAC2 card data
class ipac2(ipac_data):

    # Maps input name to index in data block - unshifted section
    map_unshifted = {
        "1UP": 1,
        "1RIGHT": 2,
        "1SW1": 3,
        "1LEFT": 4,
        "1SW3": 5,
        "1DOWN": 6,
        "1SW5": 7,
        "1SW2": 8,
        "2RIGHT": 9,
        "1SW4": 10,
        "2LEFT": 11,
        "1SW6": 12,
        "2UP": 13,
        "2DOWN": 14,
        "2SW1": 15,
        "STRT1": 16,
        "2SW2": 17,
        "STRT2": 18,
        "2SW3": 19,
        "COIN1": 20,
        "2SW4": 21,
        "COIN2": 22,
        "2SW5": 23,
        "1SW7": 24,
        "2SW6": 25,
        "1SW8": 26,
        "2SW7": 27,
        "2SW8": 28}

    # Maps input name to index in data block - shifted section
    map_shifted = {
        "1UP": 1,
        "1RIGHT": 2,
        "1SW1": 3,
        "1LEFT": 4,
        "1SW3": 5,
        "1DOWN": 6,
        "1SW5": 7,
        "1SW2": 8,
        "2RIGHT": 9,
        "1SW4": 10,
        "2LEFT": 11,
        "1SW6": 12,
        "2UP": 13,
        "2DOWN": 14,
        "2SW1": 15,
        "STRT1": 16,
        "2SW2": 17,
        "STRT2": 18,
        "2SW3": 19,
        "COIN1": 20,
        "2SW4": 21,
        "COIN2": 22,
        "2SW5": 23,
        "1SW7": 24,
        "2SW6": 25,
        "1SW8": 26,
        "2SW7": 27,
        "2SW8": 28}
    
    # List of inputs per player - ordered as in config file
    player_inputs = [
        "RIGHT", "LEFT", "UP", "DOWN", "SW1", "SW2", "SW3", "SW4",
        "SW5", "SW6", "SW7", "SW8", "STRT", "COIN"]

    player_inputs_shift = [
        "RIGHT", "LEFT", "UP", "DOWN", "SW1", "SW2", "SW3", "SW4",
        "SW5", "SW6", "SW7", "SW8", "STRT", "COIN"]

    macro_inputs = [
        "m1k1", "m1k2", "m1k3", "m1k4",
        "m2k1", "m2k2", "m2k3", "m2k4",
        "m3k1", "m3k2", "m3k3", "m3k4",
        "m4k1", "m4k2", "m4k3", "m4k4"]

    player_count = 2

    model_name = "ipac2"
    model_num  = 0
################### END IPAC2 ######################

# IPAC4 card data -- virtually the same as IPAC2
class ipac4(ipac2):
    player_count = 4

    model_name = "ipac4"
    model_num  = 1

    macro_inputs = [
        "m1k1", "m1k2", "m1k3", "m1k4",
        "m2k1", "m2k2", "m2k3", "m2k4",
        "m3k1", "m3k2", "m3k3", "m3k4",
        "m4k1", "m4k2", "m4k3", "m4k4",
        "m5k1", "m5k2", "m5k3", "m5k4",
        "m6k1", "m6k2", "m6k3", "m6k4",
        "m7k1", "m7k2", "m7k3", "m7k4",
        "m8k1", "m8k2", "m8k3", "m8k4"]


    # Is the character 1-4?
    def is_player_number(self, char):
        retval = 0
        if (char > "0") and (char < "5"):
            retval = 1
        return retval

    # Returns the offset into the data block for a specified input name
    # IE  1SW3 maps to idx 5
    # Returns -1 on error
    def get_input_index(self, input_name, shifted):
        retval = -1

        if shifted:
            map = self.map_shifted
        else:
            map = self.map_unshifted

        # Rather than including the 3/4 player table,
        # just use the 1/2 from IPAC2 with a little fixup

        offset = 0
        if ((input_name[0:1] == "3") or (input_name[0:1] == "4")):
  
            offset = len(map)
            input_name = `int(input_name[0:1]) - 2` + input_name[1:]

        elif ((self.is_player_number(input_name[0:1]) == 0) and
              (input_name[-1:] == "3") or (input_name[-1:] == "4")):

            offset = len(map)
            input_name = input_name[0:-1] + `int(input_name[-1:]) - 2`

        if (map.has_key(input_name.upper())):
            retval = map[input_name.upper()] + offset

        return retval

    def get_input_list(self, shifted):
        if shifted:
            map = self.map_shifted
        else:
            map = self.map_unshifted

        klist = map.keys()
        # Now, add player 3/4
        for input in map.keys():
            newinput = ""
            # Most inputs start with player number
            if ((input[0:1] == "1") or (input[0:1] == "2")):
                newinput = `int(input[0:1]) + 2` + input[1:]
            # But some end with player number (coin/start)
            elif ((input[-1:] == "1") or (input[-1:] == "2")):
                newinput = input[0:-1] + `int(input[-1:]) + 2`

            klist.append(newinput)


        # Now get the list sorted
        tlist = []
        for key in klist:
            tlist.append([self.get_input_index(key,shifted), key])
 
        tlist.sort()
            
        retlist = []
        for item in tlist:
            retlist.append(item[1])
            
        return retlist

################### END IPAC4 ######################

# IPACVE card data
class ipacve(ipac_data):

    # Maps input name to index in data block - unshifted section
    map_unshifted = {
        "1UP": 1,
        "1RIGHT": 2,
        "1SW1": 3,
        "1LEFT": 4,
        "1SW3": 5,
        "1DOWN": 6,
        "1SW5": 7,
        "1SW2": 8,
        "2RIGHT": 9,
        "1SW4": 10,
        "2LEFT": 11,
        "1SW6": 12,
        "2UP": 13,
        "2DOWN": 14,
        "2SW1": 15,
        "STRT1": 16,
        "2SW2": 17,
        "STRT2": 18,
        "2SW3": 19,
        "COIN1": 20,
        "2SW4": 21,
        "COIN2": 22,
        "2SW5": 23,
        "1A": 24,
        "2SW6": 25,
        "1B": 26,
        "2A": 27,
        "2B": 28,
        "1C": 29,
        "1D": 30,
        "2C": 31,
        "2D": 32}

    # Maps input name to index in data block - shifted section
    map_shifted = {
        "1UP": 1,
        "1RIGHT": 2,
        "1SW1": 3,
        "1LEFT": 4,
        "1SW3": 5,
        "1DOWN": 6,
        "1SW5": 7,
        "1SW2": 8,
        "2RIGHT": 9,
        "1SW4": 10,
        "2LEFT": 11,
        "1SW6": 12,
        "2UP": 13,
        "2DOWN": 14,
        "2SW1": 15,
        "STRT1": 16,
        "2SW2": 17,
        "STRT2": 18,
        "2SW3": 19,
        "COIN1": 20,
        "2SW4": 21,
        "COIN2": 22,
        "2SW5": 23,
        "1A": 24,
        "2SW6": 25,
        "1B": 26,
        "2A": 27,
        "2B": 28}
    
    # List of inputs per player
    player_inputs = [
        "RIGHT", "LEFT", "UP", "DOWN", "SW1", "SW2", "SW3", "SW4",
        "SW5", "SW6", "A", "B", "C", "D", "STRT", "COIN"]
    
    player_inputs_shift = [
        "RIGHT", "LEFT", "UP", "DOWN", "SW1", "SW2", "SW3", "SW4",
        "SW5", "SW6", "A", "B", "STRT", "COIN"]
    
    macro_inputs = [
        "m1k1", "m1k2", "m1k3", "m1k4",
        "m2k1", "m2k2", "m2k3", "m2k4",
        "m3k1", "m3k2", "m3k3", "m3k4",
        "m4k1", "m4k2", "m4k3", "m4k4"]

    player_count = 2

    model_name = "ipacve`"
    model_num  = 2
################### END IPACVE ######################

# MINIPACVE card data
class minipacve(ipac_data):

    # Maps input name to index in data block - unshifted section
    map_unshifted = {
        "1UP": 1,
        "1RIGHT": 2,
        "1SW1": 3,
        "1LEFT": 4,
        "1SW3": 5,
        "1DOWN": 6,
        "1SW5": 7,
        "1SW2": 8,
        "2RIGHT": 9,
        "1SW4": 10,
        "2LEFT": 11,
        "1SW6": 12,
        "2UP": 13,
        "2DOWN": 14,
        "2SW1": 15,
        "STRT1": 16,
        "2SW2": 17,
        "STRT2": 18,
        "2SW3": 19,
        "COIN1": 20,
        "2SW4": 21,
        "COIN2": 22,
        "2SW5": 23,
        "1A": 24,
        "2SW6": 25,
        "1B": 26,
        "2A": 27,
        "2B": 28,
        "1C": 29,
        "1D": 30,
        "1E": 31,
        "1F": 32,
        "2C": 33,
        "2D": 34,
        "2E": 35,
        "2F": 36}

    # Maps input name to index in data block - shifted section
    map_shifted = {
        "1UP": 1,
        "1RIGHT": 2,
        "1SW1": 3,
        "1LEFT": 4,
        "1SW3": 5,
        "1DOWN": 6,
        "1SW5": 7,
        "1SW2": 8,
        "2RIGHT": 9,
        "1SW4": 10,
        "2LEFT": 11,
        "1SW6": 12,
        "2UP": 13,
        "2DOWN": 14,
        "2SW1": 15,
        "STRT1": 16,
        "2SW2": 17,
        "STRT2": 18,
        "2SW3": 19,
        "COIN1": 20,
        "2SW4": 21,
        "COIN2": 22,
        "2SW5": 23,
        "1A": 24,
        "2SW6": 25,
        "1B": 26,
        "2A": 27,
        "2B": 28}
    
    # List of inputs per player
    player_inputs = [
        "RIGHT", "LEFT", "UP", "DOWN", "SW1", "SW2", "SW3", "SW4",
        "SW5", "SW6", "A", "B", "C", "D", "E", "F", "STRT", "COIN"]
    
    player_inputs_shift = [
        "RIGHT", "LEFT", "UP", "DOWN", "SW1", "SW2", "SW3", "SW4",
        "SW5", "SW6", "A", "B", "STRT", "COIN"]
    
    macro_inputs = [
        "m1k1", "m1k2", "m1k3", "m1k4",
        "m2k1", "m2k2", "m2k3", "m2k4",
        "m3k1", "m3k2", "m3k3", "m3k4",
        "m4k1", "m4k2", "m4k3", "m4k4"]

    player_count = 2

    model_name = "minipacve`"
    model_num  = 3
################### END MINIPACVE ######################

##################### ALL ########################
# Special class that includes all possible inputs for shifted/unsifted
# and nothing else
# all card data
class all(ipac_data):

    map_unshifted = {}
    map_shifted = {}

    # Maps input name to index in data block - unshifted section
    player_inputs = [
        "RIGHT", "LEFT", "UP", "DOWN", "SW1", "SW2", "SW3", "SW4",
        "SW5", "SW6", "SW7", "SW8", "A", "B", "C", "D", "E", "F", 
        "STRT", "COIN"]
    
    player_inputs_shift = [
        "RIGHT", "LEFT", "UP", "DOWN", "SW1", "SW2", "SW3", "SW4",
        "SW5", "SW6", "SW7", "SW8", "A", "B", "STRT", "COIN"]

    macro_inputs = [
        "m1k1", "m1k2", "m1k3", "m1k4",
        "m2k1", "m2k2", "m2k3", "m2k4",
        "m3k1", "m3k2", "m3k3", "m3k4",
        "m4k1", "m4k2", "m4k3", "m4k4",
        "m5k1", "m5k2", "m5k3", "m5k4",
        "m6k1", "m6k2", "m6k3", "m6k4",
        "m7k1", "m7k2", "m7k3", "m7k4",
        "m8k1", "m8k2", "m8k3", "m8k4"]

    player_count = 4

    model_name = "all"
    model_num  = -1

    # Returns unshifted, shifted, macro and shift code inputs
    def get_all_inputs(self):
        retVal = self.player_inputs
        for item in self.player_inputs_shift:
            retVal.append(item)
        for item in self.macro_inputs:
            retVal.append(item)
        retVal.append("shiftcode")

        return retVal

################### END ALL ######################

class ipac(ipac_data):
    
    def __init__(self, type):
        if (type == "ipac2"):
            self.child = ipac2()
        elif (type == "ipac4"):
            self.child = ipac4()
        elif (type == "ipacve"):
            self.child = ipacve()
        elif (type == "minipacve"):
            self.child = minipacve()
        elif (type == "all"):
            self.child = all()
        else:
            print "Trying to instantiate with invalid type"

    def get_player_count(self):
        try:
            return self.child.get_player_count()
        except:
            return None

    def get_macro_count(self):
        try:
            return self.child.get_macro_count()
        except:
            return None

    def get_model_name(self):
        try:
            return self.child.get_model_name()
        except:
            return None

    def get_model_num(self):
        try:
            return self.child.get_model_num()
        except:
            return None

    def get_player_inputs(self, player, shifted):
        try:
            return self.child.get_player_inputs(player, shifted)
        except:
            return None

    def get_macro_inputs(self):
        try:
            return self.child.get_macro_inputs()
        except:
            return None

    def has_input(self, input, shifted):
        try:
            return self.child.has_input(input, shifted)
        except:
            return None

    def get_input_index(self, input_name, shifted):
        try:
            return self.child.get_input_index(input_name, shifted)
        except:
            return None

    def get_input_list(self, shifted):
        #try:
        #    return self.child.get_input_list(shifted)
        #except:
        #    return None
        return self.child.get_input_list(shifted)

##########################################################################
# Key Classes - set 2 of PS/2 key scancode set
##########################################################################

class keylist:
        
    # http://www.cs.ualberta.ca/~yannis/329_06_winter/Labs/scancodes.html
    # Use Set 2
    keymap = {
        "NONE": 0x00,
        "A": 0x1C,
        "B": 0x32,
        "C": 0x21,
        "D": 0x23,
        "E": 0x24,
        "F": 0x2B,
        "G": 0x34,
        "H": 0x33,
        "I": 0x43,
        "J": 0x3B,
        "K": 0x42,
        "L": 0x4B,
        "M": 0x3A,
        "N": 0x31,
        "O": 0x44,
        "P": 0x4D,
        "Q": 0x15,
        "R": 0x2D,
        "S": 0x1B,
        "T": 0x2C,
        "U": 0x3C,
        "V": 0x2A,
        "W": 0x1D,
        "X": 0x22,
        "Y": 0x35,
        "Z": 0x1A,
        "0": 0x45,
        "1": 0x16,
        "2": 0x1E,
        "3": 0x26,
        "4": 0x25,
        "5": 0x2E,
        "6": 0x36,
        "7": 0x3D,
        "8": 0x3E,
        "9": 0x46,
        "`": 0x0E,
        "-": 0x4E,
        "=": 0x55,
        ",": 0x41,
        "\\": 0x5D,
        "BKSP": 0x66,
        "SPACE": 0x29,
        "TAB": 0x0D,
        "CAPS": 0x58,
        "LSHFT": 0x12,
        "LCTRL": 0x14,
        "LALT": 0x11,
        #LWIN is E0, 1F
        "LWIN": 0x9F,
        "RSHFT": 0x59,
        # RCTRL is E0 14
        "RCTRL": 0x94,
        # RALT is E0, 11
        "RALT": 0x91,
        # RWIN is E0, 27
        "RWIN": 0xC7,
        # APPS is E0, 2F
        "APPS": 0xCF,
        "ENTER": 0x5A,
        "ESC": 0x76,
        "F1": 0x05,
        "F2": 0x06,
        "F3": 0x04,
        "F4": 0x0C,
        "F5": 0x03,
        "F6": 0x0B,
        "F7": 0x83,
        "F8": 0x0A,
        "F9": 0x01,
        "F10": 0x09,
        "F11": 0x78,
        "F12": 0x07,
        #Prt scr is 7C/84
        #"PRNTSCRN": 0x7C,
        "SCROLL": 0x7E,
        #Pause is 77/7E
        #"PAUSE": 0xE1,
        "[": 0x54,
        # Insert is E0,70
        "INSERT": 0xF0,
        #Home is E0,6C
        "HOME": 0xEC,
        #PGUP is E0,7D
        "PGUP": 0xFD,
        #Delete is E0, 71
        "DELETE": 0xF1,
        #End is E0, 69
        "END": 0xE9,
        # PGDN is E0, 7A
        "PGDN": 0xFA,
        #UP is E0, 75
        "UARROW": 0xF5,
        #left is E0,6B
        "LARROW": 0xEB,
        #down is E0,72
        "DARROW": 0xF2,
        #right is E0,74
        "RARROW": 0xF4,
        "NUM": 0x77,
        #keypad / is E0, 4A
        "KP/": 0xCA,
        "KP*": 0x7C,
        "KP-": 0x7B,
        "KP+": 0x79,
        # Keypad enter is E0, 5A
        "KPEN": 0xDA,
        "KP.": 0x71,
        "KP0": 0x70,
        "KP1": 0x69,
        "KP2": 0x72,
        "KP3": 0x7A,
        "KP4": 0x6B,
        "KP5": 0x73,
        "KP6": 0x74,
        "KP7": 0x6C,
        "KP8": 0x75,
        "KP9": 0x7D,
        "]": 0x5B,
        ";": 0x4C,
        "'": 0x52,
        ".": 0x49,
        "/": 0x4A,
        "DEF_SHIFT": 0x10}

    def get_keys(self):
        #return self.keymap.keys()
        lst = self.keymap.keys()
        lst.sort()
        return lst
        
    def get_keycode(self, keyname):
        return self.keymap[keyname.upper()]

    def get_keyname(self, keycode):
        #print "get_keyname: %s" % `keycode`
        tdict = {}
        for key in self.keymap.keys():
            tdict[self.keymap[key]] = key
            #print "setting tdict[%s] = %s" % (self.keymap[key], key)

        if (keycode[0:2] == "0x"):
            base = 16
        else:
            base = 10
        keycode = int(keycode,base)
        #print "get_keyname: keycode: %d = %s" % (keycode, tdict[keycode])

        return tdict[keycode]

    # Gets the offset of a keycode from the get_keys list
    def get_keyname_offset(self, keyname):
        lst = self.get_keys()
        return lst.index(keyname)
